﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Collide: MonoBehaviour
{
  //TODO: collision impulse may need to be used instead of merely relative velocity? Or perhaps
  //      relative velocity projected onto normal?
  [Tooltip("Magnitude magnitude of impulse force when hit by wrecking ball")]
  public float maxImpulseMagnitude = 5;

  [Tooltip("Relative velocity of colliding bodies at which impulse magnitude reaches its maximum value (LERPed below this).")]
  public float relativeVelocityThreshold = 2;

  private void OnCollisionEnter(Collision collision)
  {
    GameObject other = collision.collider.gameObject;
    if (other.name == "WreckingBall")
    {
      Rigidbody rb = GetComponent<Rigidbody>();
      Vector3 direction = collision.contacts[0].normal;
      float relativeVelocity = collision.relativeVelocity.magnitude;
      float impulseMagnitude = Mathf.Lerp(0, maxImpulseMagnitude, relativeVelocity / relativeVelocityThreshold);
      rb.AddForce(direction * impulseMagnitude, ForceMode.Impulse);
    }
  }
}
