﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.VR.WSA;
using UnityEngine.VR.WSA.Input;

public class GameController: MonoBehaviour
{
  [Tooltip("A solid cube, just for testing.")]
  public GameObject cubePrefab;

  [Tooltip("Enemy robot.")]
  public GameObject robotPrefab;

  [Tooltip("Objects to set stabilization plane at (average position is used).")]
  public GameObject[] stabilizationTargets;

  enum State
  {
    Init,
    Scanning,
    FinalizeScan,
    Playing
  }

  private GestureRecognizer m_gestureRecognizer;
  private State m_state = State.Init;

  private void PlaceObjects()
  {
    Vector3 position;

    // A couple of test cubes
    if (PlayspaceManager.Instance.TryPlaceOnFloor(out position, cubePrefab.transform.localScale))
      Instantiate(cubePrefab, position, Quaternion.identity);
    if (PlayspaceManager.Instance.TryPlaceOnPlatform(out position, 0.25f, 1.5f, cubePrefab.transform.localScale.x))
      Instantiate(cubePrefab, position + cubePrefab.transform.localScale.y * Vector3.up, Quaternion.identity);

    // Chibi-robot!
    //if (PlayspaceManager.Instance.TryPlaceOnPlatform(out position, 0.25f, 1.5f, 0.25f))
    //  Instantiate(robotPrefab, position, Quaternion.identity);
  }

  private void SetState(State state)
  {
    m_state = state;
    switch (state)
    {
      case State.Scanning:
        Debug.Log("State: Scanning");
        PlayspaceManager.Instance.StartScanning();
        break;
      case State.FinalizeScan:
        Debug.Log("State: FinalizeScan");
        System.Action OnScanComplete = () =>
        {
          SetState(State.Playing);
        };
        PlayspaceManager.Instance.OnScanComplete += OnScanComplete;
        PlayspaceManager.Instance.StopScanning();
        break;
      case State.Playing:
        Debug.Log("State: Playing");
        PlaceObjects();
        break;
    }
  }

  private void OnTapEvent(InteractionSourceKind source, int tapCount, Ray headRay)
  {
    switch (m_state)
    {
      case State.Scanning:
        SetState(State.FinalizeScan);
        break;
      case State.Playing:
        break;
    }
  }

#if UNITY_EDITOR
  private void Update()
  {
    // Simulate air tap
    if (Input.GetKeyDown(KeyCode.Return))
    {
      Ray headRay = new Ray(transform.position, transform.forward);
      OnTapEvent(InteractionSourceKind.Hand, 1, headRay);
    }
  }
#endif

  private void LateUpdate()
  {
    //GameObject target = stabilizationTargets[0];
    HolographicSettings.SetFocusPointForFrame(Camera.main.transform.position + Camera.main.transform.forward * 1f, -Camera.main.transform.forward);
  }

  private void Start()
  {
    m_gestureRecognizer = new GestureRecognizer();
    m_gestureRecognizer.SetRecognizableGestures(GestureSettings.Tap);
    m_gestureRecognizer.TappedEvent += OnTapEvent;
    m_gestureRecognizer.StartCapturingGestures();
    SetState(State.Scanning);
  }
}
